import logging
import os
from bn import AttributeDict

log = logging.getLogger(__name__)

class HTTPObject(AttributeDict):
    def __init__(
        self, 
        flow,
        default_status='200 OK', 
        default_headers=[('Content-Type', 'text/html; charset=utf-8')],
    ):
        self.status = default_status
        self.headers = [x for x in default_headers]
        self.body = []
        self['_result_returned'] = False
        self['_flow'] = flow
        self['_result_returned'] = False

    def __setattr__(self, name, value):
        if name in ['status', 'headers', 'body']:
            self[name] = value
        else:
            AttributeDict.__setattr__(self, name, value)

    def get_result(self):
        if self._result_returned:
            raise Exception(
                'The result has already been returned, you cannot get it '
                'again'
            )
        self['_result_returned'] = True
        if isinstance(self.body, list) and self.body.__class__.__bases__ == (object,):
            return ResultIterator(self.body)
        elif isinstance(self.body, (unicode, str)):
            log.warning('String value found as response, converting to a list')
            return ResultIterator([self.body])
        return self.body

class ResultIterator:
    def __init__(self, body, charset='utf8'):
        self.body = body
        if not hasattr(self.body, '__iter__') and not \
           isinstance(self.body, list):
            raise TypeError(
                'The body must be a list of strings and iterables, '
                'not %r'%type(body)
            )
        self.index = -1
        self.charset = charset
        self.body_length = len(self.body)

    def __iter__(self):
        return self

    def next(self):
        if self.index+2 > self.body_length:
            raise StopIteration
        self.index += 1
        part = self.body[self.index]
        # This isn't technically allowed, but let's try to be nice
        if isinstance(part, list):
            log.warning('List found in response when Unicode string expected')
            return (''.join(part)).encode(self.charset)
        else:
            return part.encode(self.charset)



class HTTPService(object):

    @staticmethod
    def create(flow, name, config=None):
        return HTTPService()

    @staticmethod
    def config(flow, name):
        # There is no configuration
        return None

    def start(self, flow, name):
        flow[name] = AttributeDict()
        flow[name]['response'] = HTTPObject(flow)
        flow[name]['environ'] = None
        

